@@ -252,12 +252,17 @@ create_plug (BonoboControl *control)
 	
 	plug = bonobo_plug_new (0);
 	gtk_widget_show (plug);
+
+	g_object_ref (G_OBJECT (plug));
+	gtk_object_sink (GTK_OBJECT (plug));
 	
 	bonobo_control_set_plug (control, BONOBO_PLUG (plug));
 
 	if (control->priv->widget)
 		gtk_container_add (GTK_CONTAINER (plug),
 				   control->priv->widget);
+
+	g_object_unref (G_OBJECT (plug));
 }
 
 static CORBA_char *
@@ -334,6 +339,14 @@ impl_Bonobo_Control_setSize (PortableServer_Servant  servant,
 			     const CORBA_short       height,
 			     CORBA_Environment      *ev)
 {
+	GtkAllocation  size;
+	BonoboControl *control = BONOBO_CONTROL (
+		bonobo_object_from_servant (servant));
+	
+	size.x = size.y = 0;
+	size.width = width;
+	size.height = height;
+
 	/*
 	 * Nothing.
 	 *
@@ -341,6 +354,10 @@ impl_Bonobo_Control_setSize (PortableServer_Servant  servant,
 	 * is handled by GtkPlug/GtkSocket for us, or GtkFrame in the
 	 * local case.
 	 */
+
+	g_warning ("setSize untested");
+
+	gtk_widget_size_allocate (GTK_WIDGET (control->priv->plug), &size);
 }
 
 static Bonobo_Gtk_Requisition
@@ -584,8 +601,10 @@ bonobo_control_destroy (BonoboObject *object)
 	bonobo_control_set_properties      (control, CORBA_OBJECT_NIL, NULL);
 	bonobo_control_set_ui_component    (control, NULL);
 
-	if (control->priv->widget)
+	if (control->priv->widget) {
+		gtk_widget_destroy (GTK_WIDGET (control->priv->widget));
 		g_object_unref (control->priv->widget);
+	}
 	control->priv->widget = NULL;
 
 	control->priv->popup_ui_container = bonobo_object_unref (
@@ -1108,7 +1127,7 @@ bonobo_control_set_transient_for (BonoboControl     *control,
 	}
 
 	id = Bonobo_ControlFrame_getToplevelId (frame, ev);
-	g_return_if_fail (id != NULL);
+	g_return_if_fail (!BONOBO_EX (ev) && id != NULL);
 
 	x11_id = bonobo_control_x11_from_window_id (id);
 
@@ -1163,18 +1182,24 @@ bonobo_control_set_plug (BonoboControl *control,
 	if ((BonoboPlug *) control->priv->plug == plug)
 		return;
 
+	dprintf ("bonobo_control_set_plug (%p, %p) [%p]\n",
+		 control, plug, control->priv->plug);
+
 	old_plug = (BonoboPlug *) control->priv->plug;
 
-	if (plug) {
+	if (plug)
 		control->priv->plug = g_object_ref (plug);
-		bonobo_plug_set_control (plug, control);
-	} else
+	else
 		control->priv->plug = NULL;
 
 	if (old_plug) {
 		bonobo_plug_set_control (old_plug, NULL);
+		gtk_widget_destroy (GTK_WIDGET (old_plug));
 		g_object_unref (old_plug);
 	}
+
+	if (plug)
+		bonobo_plug_set_control (plug, control);
 }
 
 BonoboPlug *
